package refactor;

import mixin.AST_TypeName;
import mixin.AstNode;
import mixin.DecNameDim;
import mixin.VarDecl;

import java.io.StringReader;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import mixin.ArrInit;
import mixin.AsgExpr;
import mixin.AST_Stmt;
import mixin.AST_VarInit;
import mixin.AstOptNode;
import mixin.BlockStmt;
import mixin.ExprStmt;
import mixin.ParseException;
import mixin.Parser;
import mixin.VarAssignC;
import mixin.VarInitExpr;




/**
 * Implementierung von FieldInfo, die den AST zur Ermittlung der Informationen
 * nutzt.
 * 
 * @layer<typesys>
 * @author Stephan Klapproth
 * 
 */
abstract class FieldInfoAst$$typesys extends AbstractNamed implements FieldInfo, AstElement {

    private ClassInfo type;

    private AST_TypeName astTypeName;

    private VarDecl varDecl;
    
    private DecNameDim decNameDim;

    private Scope context;
    
    /**
     * Konstruktor.
     * 
     * @param aTypeName
     *            AstNode, der den Typ der Variablen beschreibt.
     * @param aVarDecl
     *            AstNode, der die Variablendeklaration beschreibt.
     */
    public FieldInfoAst$$typesys(AST_TypeName aTypeName, VarDecl aVarDecl) {
        this(aTypeName, (DecNameDim) aVarDecl.getVariableDeclaratorId());
        varDecl = aVarDecl;
    }
    
    /**
     * Konstruktor.
     * 
     * @param aTypeName
     *            AstNode, der den Typ der Variablen beschreibt.
     * @param aDecNameDim
     *            AstNode, der den Namen der Variablen beinhaltet.
     */
    public FieldInfoAst$$typesys(AST_TypeName aTypeName, DecNameDim aDecNameDim) {
        astTypeName = aTypeName;
        decNameDim = aDecNameDim;
        initName();
        type = null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see FieldInfo#getType()
     */
    public ClassInfo getType() {
        if (type == null) {
            int tmpDim = ArrayType.getDimCount(astTypeName);
            tmpDim = tmpDim
                    + ArrayType.getDimCount(decNameDim);
            type = TypeFactory.valueOf(astTypeName.GetName(), getContext(),
                    tmpDim);
        }
        return type;
    }
    
    VarDecl getVarDecl() {
        return varDecl;
    }
    
    private void initName() {
        name = decNameDim.getQName()
                .GetName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see Declaration#getContext()
     */
    public Scope getContext() {
        return context;
    }

    /*
     * (non-Javadoc)
     * 
     * @see Declaration#setContext(Scope)
     */
    public void setContext(Scope scope) {
        context = scope;
    }

    /*
     * (non-Javadoc)
     * 
     * @see AbstractNamed#getFullName()
     */
    public String getFullName() {
        if (context == null) {
            throw new IllegalStateException("Declaring scope for " + getName() + " not set.");
        }
        return context.getFullName() + TypesysUtil.DOT + getName();
    }

    public int endLineNum() {
        return decNameDim.findToken().lineNum();
    }

    public AstNode getAstNode() {
        return decNameDim;
    }

    public int startLineNum() {
        return decNameDim.findToken().lineNum();
    }
    

    /**
     * Gibt zurck, ob es sich um eine lokale Variable (Methodenparameter oder
     * temporre Variable) handelt.
     * 
     * @return <code>true</code>, wenn es sich um eine lokale Variable
     *         handelt.
     */
    public boolean isLocalVar() {
        return !isAttribute();
    }

    /**
     * Gibt zurck, ob es sich um Objektattribut handelt.
     * 
     * @return <code>true</code>, wenn es sich um ein Objektattribut handelt.
     */
    public boolean isAttribute() {
        return getContext() instanceof ClassInfoAst;
    }

    /**
     * Gibt zurck, ob es sich um einen Methodenparameter handelt.
     * 
     * @return <code>true</code>, wenn es sich um einen Methodenparameter
     *         handelt.
     */
    public boolean isMethodParam() {
        return getContext() instanceof MethodInfoAst;
    }

    /**
     * Gibt zurck, ob es sich um eine temporre Variable handelt.
     * 
     * @return <code>true</code>, wenn es sich um eine temporre Variable
     *         handelt.
     */
    public boolean isTemporaryVar() {
        return !(isAttribute() || isMethodParam());
    }
}



/**
 * @layer<refactoring_extrstmts>
 * @author Stephan Klapproth
 * 
 */
 class FieldInfoAst extends  FieldInfoAst$$typesys  {
    
    private static final Log LOG = LogFactory
            .getLog("refactoring.extrstmts.FieldInfoAst");

    boolean isInitialized() {
        return ((VarDecl) getAstNode().up).getVarAssign() instanceof VarAssignC;
    }

    void delete() {
        // Statement lschen
        BlockStmt tmpBlockStmt = getBlockStmt(getAstNode());
        if (tmpBlockStmt != null) {
            tmpBlockStmt.Delete();
        }
    }

    void transformToVarAssign() {
        AST_VarInit tmpInit = ((VarAssignC) getVarDecl().getVarAssign())
                .getAST_VarInit();

        AST_Stmt tmpStmt = createVarAssign(tmpInit);
        if (tmpStmt == null) {
            // nicht untersttztes VarInit (ViEscape)
            return;
        }
        ExprStmt tmpAsgExprStmt = (ExprStmt) tmpStmt.arg[0].arg[0];
        int tmpLineNum = tmpInit.findToken().lineNum();
        correctLineNums(tmpLineNum, tmpAsgExprStmt);
        // alte Variablendeklaration durch Variablenzuweisung ersetzen
        BlockStmt tmpBlockStmt = getBlockStmt(getAstNode());
        if (tmpBlockStmt != null) {
            tmpBlockStmt.Replace(tmpAsgExprStmt);
        }
    }
    
    private void correctLineNums(int aLineNum, ExprStmt anExpr) {
        RefUtil.setLineNum(aLineNum, anExpr.findToken());
        RefUtil.setLineNum(aLineNum, TypesysUtil.findLastToken(anExpr));
        
        AsgExpr tmpAsg = (AsgExpr) anExpr.getAST_Exp();
        RefUtil.setLineNum(aLineNum, tmpAsg.getConditionalExpression()
                .findToken());
        RefUtil.setLineNum(aLineNum, tmpAsg.getAssignmentOperator()
                .findToken());
        RefUtil.setLineNum(aLineNum, tmpAsg.getExpression()
                .findToken());
    }

    private AST_Stmt createVarAssign(AST_VarInit aVarInit) {
        StringBuilder tmpBuilder = new StringBuilder();
        tmpBuilder.append("\n");
        tmpBuilder.append(RefUtil.indent(2));
        tmpBuilder.append(getName());
        tmpBuilder.append(" = ");
        if (aVarInit instanceof VarInitExpr) {
            tmpBuilder.append(((VarInitExpr) aVarInit).getExpression());
        } else if (aVarInit instanceof ArrInit) {
            tmpBuilder.append("new ").append(getType().getFullName()).append(
                    aVarInit);
        } else {
            return null;
        }
        tmpBuilder.append(";");
        return AST_Stmt.MakeAST(tmpBuilder.toString());
    }

    private BlockStmt getBlockStmt(AstNode aNode) {
        AstNode tmpBlockStmt = aNode;
        while (tmpBlockStmt != null && !(tmpBlockStmt instanceof BlockStmt)) {
            tmpBlockStmt = tmpBlockStmt.up;
        }
        return (BlockStmt) tmpBlockStmt;
    }

    void setVarInit(String anInitExpr) throws TransformationException {
        Parser tmpParser = Parser.getInstance(new StringReader(" = "
                + anInitExpr));
        try {
            AstOptNode anAssign = new AstOptNode().setParms(tmpParser
                    .parse("VarAssign"));
            getVarDecl().setParms(getVarDecl().getVariableDeclaratorId(),
                    anAssign);
        } catch (ParseException e) {
            LOG.error(e);
            throw new TransformationException(e);
        }
    }
      // inherited constructors


    
    /**
     * Konstruktor.
     * 
     * @param aTypeName
     *            AstNode, der den Typ der Variablen beschreibt.
     * @param aDecNameDim
     *            AstNode, der den Namen der Variablen beinhaltet.
     */
    public FieldInfoAst ( AST_TypeName aTypeName, DecNameDim aDecNameDim ) { super(aTypeName, aDecNameDim); }
    
    /**
     * Konstruktor.
     * 
     * @param aTypeName
     *            AstNode, der den Typ der Variablen beschreibt.
     * @param aVarDecl
     *            AstNode, der die Variablendeklaration beschreibt.
     */
    public FieldInfoAst ( AST_TypeName aTypeName, VarDecl aVarDecl ) { super(aTypeName, aVarDecl); }
}
