package refactor;



/**
 * Dieses Interface beschreibt einen Layer.
 * <p>
 * Jeder Layer hat einen Verweis auf seinen Vorgnger und seinen Nachfolger in
 * der Equation.
 * 
 * @layer<typesys>
 * @author Stephan Klapproth
 * @author Stephan Kauschka
 */
public interface LayerInfo extends Scope {

    /**
     * Jedes Layer wird von genau einem Typsystem verwaltet.
     * 
     * @return das dem Layer assoziierte Typsystem.
     */
    public TypeSystem getTypeSystem();

    /**
     * Gibt alle von diesem Layer definierten Klassen zur&uuml;ck.
     * 
     * @return Liste, mit Elementen vom Typ {@link ClassInfo}.
     */
    public ClassInfo[] getClasses();

    /**
     * Gibt den ersten Layer der Equation zurck.
     * 
     * @return erster Layer.
     */
    public LayerInfo first();

    /**
     * Gibt den nchsten Layer in der Equation zurck.
     * <p>
     * Wenn es sich bei dem Layer bereits um den letzten handelt, wird
     * <code>null</code> zurckgegeben.
     * 
     * @return nchster Layer.
     */
    public LayerInfo getNext();

    /**
     * ndert den Verweis auf den nchsten Layer.
     * 
     * @param aLayer
     */
    public void setNext(LayerInfo aLayer);

    /**
     * Gibt den vorigen Layer in der Equation zurck.
     * 
     * @return
     */
    public LayerInfo getPrev();

    /**
     * ndert den Verweis auf den vorigen Layer.
     * 
     * @param aLayer
     */
    public void setPrev(LayerInfo aLayer);

    /**
     * Sucht nach der Klasse mit dem Namen <code>aName</code>.
     * <p>
     * Wenn die Klasse in dem Layer nicht gefunden wird, wird in den vorherigen
     * Layern nach der Klasse gesucht.
     * <p>
     * Schlgt die Suche fehl, wird <code>null</code> zurckgegeben.
     * 
     * @param aName
     *            Name der Klasse.
     * @return
     */
    public ClassInfo findClass(String aName);

}
